package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/enum"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"strconv"
	"time"
	"xorm.io/xorm"
)

func Init(redisAddr string) (err error) {
	if redisAddr != "" {
		cache.NewRedis(redisAddr)
	}
	_, err = cache.SelectDb(md.RedisDataBase)
	return
}

// DailySettlementBlockStarChain 每日结算“区块星链”
func DailySettlementBlockStarChain(engine *xorm.Engine, mid string, isTask bool) (err error) {
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()
	now := time.Now()
	today := now.Format("2006-01-02")

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", now.Hour())
	if isTask && (now.Hour() > 8 || now.Hour() < 1) {
		//TODO::只在凌晨一点 ~ 凌晨 8 点运行
		return errors.New("非运行时间")
	}
	//1、查找 `block_star_chain` 基础设置
	blockStarChain, err := db.BlockStarChainGetOneByParams(session, map[string]interface{}{
		"key":   "is_use",
		"value": 1,
	})
	if err != nil {
		_ = session.Rollback()
		return err
	}
	if blockStarChain.SettlementDate == today {
		_ = session.Rollback()
		return errors.New("今日“区块星链”已结算")
	}
	//initialCoinTotal := zhios_order_relate_utils.StrToFloat64(blockStarChain.InitialCoinTotal)             //初始区块币总量
	todayPublishCoin := zhios_order_relate_utils.StrToFloat64(blockStarChain.TodayPublishCoin)             //今日区块币发行数量(若为0,则按当前每日区块币发行数量)
	nowEverydayPublishCoin := zhios_order_relate_utils.StrToFloat64(blockStarChain.NowEverydayPublishCoin) //当前每日区块币发行数量

	//2、判断今日是否有系统销毁
	var destroyCoinForSystem string
	if todayPublishCoin != 0 && todayPublishCoin != nowEverydayPublishCoin {
		destroyCoinForSystem, _ := decimal.NewFromFloat(nowEverydayPublishCoin).Sub(decimal.NewFromFloat(todayPublishCoin)).Float64()
		if destroyCoinForSystem < 0 {
			_ = session.Rollback()
			return errors.New("当前\"今日区块币发行数量\"大于\"当前每日区块币发行数量\"")
		}
		err := DealDestroyCoin(session, int(enum.SystemDestroy), destroyCoinForSystem, enum.SystemDestroy.String(), blockStarChain)
		if err != nil {
			_ = session.Rollback()
			return err
		}
	}
	var publishCoin = nowEverydayPublishCoin

	//3、进行系统区块币发放
	err = dealIssueCoin(session, int(enum.SystemTimingIssue), publishCoin, enum.SystemTimingIssue.String(), blockStarChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	userIds, err := db.UserLevelDescByWeightLow(session)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//4、进行静态区-区块币统计分配
	var staticAreaCoinNums float64                                                          //静态区分配区块币总数
	publishCoinStaticRate, _ := decimal.NewFromString(blockStarChain.PublishCoinStaticRate) //区块币发行静态占比
	publishCoinStaticRate = publishCoinStaticRate.Div(decimal.NewFromFloat(100))
	staticAreaCoinNums = zhios_order_relate_utils.StrToFloat64(decimal.NewFromFloat(publishCoin).Mul(publishCoinStaticRate).String())
	err, destroyCoinForStatic := statisticsAndDistributeCoinForStatic(session, userIds, mid, staticAreaCoinNums, *blockStarChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//5、进行动态区-区块币统计分配
	var dynamicAreaCoinNums float64                                                           //动态区分配区块币总数
	publishCoinDynamicRate, _ := decimal.NewFromString(blockStarChain.PublishCoinDynamicRate) //区块币发行动态占比
	publishCoinDynamicRate = publishCoinDynamicRate.Div(decimal.NewFromFloat(100))
	dynamicAreaCoinNums = zhios_order_relate_utils.StrToFloat64(decimal.NewFromFloat(publishCoin).Mul(publishCoinDynamicRate).String())
	err, destroyCoinForDynamic := statisticsAndDistributeCoinForDynamic(session, userIds, mid, dynamicAreaCoinNums, *blockStarChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//6、进行运营中心-区块币统计 TODO::未完成分配
	var operationCenterCoinNums float64                                                                       //运营中心-分配区块币总数
	publishCoinOperationCenterRate, _ := decimal.NewFromString(blockStarChain.PublishCoinOperationCenterRate) //区块币发行运营中心占比
	publishCoinOperationCenterRate = publishCoinOperationCenterRate.Div(decimal.NewFromFloat(100))
	operationCenterCoinNums = zhios_order_relate_utils.StrToFloat64(decimal.NewFromFloat(publishCoin).Mul(publishCoinOperationCenterRate).String())
	err, destroyCoinForOperationCenter := statisticsAndDistributeCoinForOperationCenter(session, mid, operationCenterCoinNums, *blockStarChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//7、进行其他-区块币统计 TODO::未完成分配
	var otherCoinNums float64                                                                      //其他 分配区块币总数
	publishCoinOperationOtherRate, _ := decimal.NewFromString(blockStarChain.PublishCoinOtherRate) //区块币发行 其他 占比
	publishCoinOperationOtherRate = publishCoinOperationOtherRate.Div(decimal.NewFromFloat(100))
	otherCoinNums = zhios_order_relate_utils.StrToFloat64(decimal.NewFromFloat(publishCoin).Mul(publishCoinOperationOtherRate).String())
	err, destroyCoinForOther := statisticsAndDistributeCoinForOther(session, mid, otherCoinNums, *blockStarChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//8、插入 block_star_chain_settlement_records 记录
	var blockStarChainSettlementRecords = model.BlockStarChainSettlementRecords{
		CoinId:                             blockStarChain.Coin1,
		TodayPublishCoin:                   zhios_order_relate_utils.Float64ToStr(publishCoin),
		TodayDestroyCoinForSystem:          destroyCoinForSystem,
		TodayPublishCoinForStatic:          zhios_order_relate_utils.Float64ToStr(staticAreaCoinNums),
		TodayDestroyCoinForStatic:          zhios_order_relate_utils.Float64ToStr(destroyCoinForStatic),
		TodayPublishCoinForDynamic:         zhios_order_relate_utils.Float64ToStr(dynamicAreaCoinNums),
		TodayDestroyCoinForDynamic:         zhios_order_relate_utils.Float64ToStr(destroyCoinForDynamic),
		TodayPublishCoinForOperationCenter: zhios_order_relate_utils.Float64ToStr(operationCenterCoinNums),
		TodayDestroyCoinForOperationCenter: zhios_order_relate_utils.Float64ToStr(destroyCoinForOperationCenter),
		TodayPublishCoinForOther:           zhios_order_relate_utils.Float64ToStr(otherCoinNums),
		TodayDestroyCoinForOther:           zhios_order_relate_utils.Float64ToStr(destroyCoinForOther),
		Date:                               now.Format("2006-01-02"),
		CreateAt:                           now,
		UpdateAt:                           now,
	}
	_, err = db.BlockStarChainSettlementRecordsInsert(session, &blockStarChainSettlementRecords)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//9、计算当前每日区块币应发行数量
	err = calcNowEverydayPublishCoin(session, *blockStarChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//10、更新 block_star_chain 中的 settlement_date 字段
	blockStarChain.SettlementDate = today
	updateAffected, err := db.BlockStarChainUpdate(session, blockStarChain.Id, blockStarChain, "settlement_date")
	if err != nil {
		_ = session.Rollback()
		return err
	}
	if updateAffected == 0 {
		_ = session.Rollback()
		return errors.New("更新 block_star_chain 的 settlement_date 记录失败")
	}

	//11、将昨日的“平台区块币指导价”插入到 block_coin_platform_guide_price_for_coin_records
	var blockCoinPlatformGuidePriceForCoinRecords = model.BlockCoinPlatformGuidePriceForCoinRecords{
		CoinId:                    blockStarChain.Coin1,
		PlatformGuidePriceForCoin: blockStarChain.PlatformGuidePriceForCoin,
		Date:                      now.AddDate(0, 0, -1).Format("2006-01-02"),
		CreateAt:                  now,
		UpdateAt:                  now,
	}
	_, err = db.BlockCoinPlatformGuidePriceForCoinRecordsInsert(session, &blockCoinPlatformGuidePriceForCoinRecords)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败")
	}
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>区块星链结束<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")
	return nil
}

/*
   统计分配区块币-静态区
   TODO:: 公式【 个人贡献值/全网贡献值x2100枚=每天获取的惠积分 】
*/
func statisticsAndDistributeCoinForStatic(session *xorm.Session, userIds []int, mid string, publishCoin float64, chain model.BlockStarChain) (err error, unassignedTotalCoinValue float64) {
	publishCoinValue := decimal.NewFromFloat(publishCoin)                                                                          //静态区发行区块币数量
	platformGuidePriceForCoinValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(chain.PlatformGuidePriceForCoin)) //今日平台区块币指导价
	var unassignedTotalCoin = decimal.NewFromFloat(0)                                                                              //未分配完的区块币
	var userVirtualAmount model.UserVirtualAmount
	var userVirtualAmounts []model.UserVirtualAmount
	//1、统计出静态区总贡献值
	sumStatic, err := session.Table("user_virtual_amount").Where("coin_id =?", chain.Coin2).Sum(&userVirtualAmount, "amount")
	if err != nil {
		return err, unassignedTotalCoinValue
	}
	sumStaticValue := decimal.NewFromFloat(sumStatic)

	//2、查询出所有拥有静态贡献值的用户
	err = session.Table("user_virtual_amount").NotIn("uid", userIds).Where("coin_id =?", chain.Coin2).And("amount > 0").Find(&userVirtualAmounts)
	if err != nil {
		return err, unassignedTotalCoinValue
	}

	for _, item := range userVirtualAmounts {
		amount := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(item.Amount)) //用户贡献值余额
		getCoin := amount.Div(sumStaticValue).Mul(publishCoinValue)                        //得到的区块币
		needDestroyContribution := getCoin.Mul(platformGuidePriceForCoinValue)             //需销毁贡献值

		coinAmount, err := svc.GetUserCoinAmount(session, mid, chain.Coin2, item.Uid) //获取此时用户的"静态贡献值"
		if err != nil {
			return err, unassignedTotalCoinValue
		}
		coinAmountValue, _ := decimal.NewFromString(coinAmount)

		//3.1判断静态贡献值是否足够
		if needDestroyContribution.GreaterThan(coinAmountValue) {
			//TODO::公式【得到的区块币 = ((需销毁贡献值 - 用户贡献值余额) / 今日平台区块币指导价)】
			tempCoin := (needDestroyContribution.Sub(coinAmountValue)).Div(platformGuidePriceForCoinValue)
			getCoin = getCoin.Sub(tempCoin)
			unassignedTotalCoin.Add(tempCoin)
			needDestroyContribution = coinAmountValue
		}

		//3.2给相应用户加上分配到的虚拟币
		err = DealUserCoin(session, md.DealUserCoinReq{
			Kind:         "add",
			Mid:          mid,
			Title:        md.StaticAreaDistributionTitleForUserVirtualCoinFlow,
			TransferType: md.StaticAreaDistributionTransferTypeForUserVirtualCoinFlow,
			OrdId:        "",
			CoinId:       chain.Coin1,
			Uid:          item.Uid,
			Amount:       zhios_order_relate_utils.StrToFloat64(getCoin.String()),
		})
		if err != nil {
			return err, unassignedTotalCoinValue
		}

		//3.3给相应用户扣除 "静态发放(销毁静态贡献值)"
		err = DealUserCoin(session, md.DealUserCoinReq{
			Kind:         "sub",
			Mid:          mid,
			Title:        md.StaticIssueAndDestroyStaticContributionTitleForUserVirtualCoinFlow,
			TransferType: md.StaticIssueAndDestroyStaticContributionTransferTypeForUserVirtualCoinFlow,
			OrdId:        "",
			CoinId:       chain.Coin2,
			Uid:          item.Uid,
			Amount:       zhios_order_relate_utils.StrToFloat64(needDestroyContribution.String()),
		})
		if err != nil {
			return err, unassignedTotalCoinValue
		}
	}

	//4、处理未分配完的区块币-静态区
	unassignedTotalCoinValue = zhios_order_relate_utils.StrToFloat64(unassignedTotalCoin.String())
	if unassignedTotalCoinValue > 0 {
		err := DealDestroyCoin(session, int(enum.StaticUnallocatedAndDestroy), unassignedTotalCoinValue, enum.StaticUnallocatedAndDestroy.String(), &chain)
		if err != nil {
			return err, unassignedTotalCoinValue
		}
	}
	return nil, unassignedTotalCoinValue
}

/*
   统计分配区块币-动态区
*/
func statisticsAndDistributeCoinForDynamic(session *xorm.Session, userIds []int, mid string, publishCoin float64, chain model.BlockStarChain) (err error, unassignedTotalCoinValue float64) {
	publishCoinValue := decimal.NewFromFloat(publishCoin)                                                                          //动态区发行区块币数量
	platformGuidePriceForCoinValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(chain.PlatformGuidePriceForCoin)) //今日平台区块币指导价
	var unassignedTotalCoin = decimal.NewFromFloat(0)                                                                              //未分配完的区块币
	var userVirtualAmount model.UserVirtualAmount
	var userVirtualAmounts []model.UserVirtualAmount

	//1、统计出动态区总贡献值
	sumStatic, err := session.Table("user_virtual_amount").Where("coin_id =?", chain.Coin3).Sum(&userVirtualAmount, "amount")
	if err != nil {
		return err, unassignedTotalCoinValue
	}
	sumStaticValue := decimal.NewFromFloat(sumStatic)

	//2、查询出所有拥有静态贡献值的用户
	err = session.Table("user_virtual_amount").NotIn("uid", userIds).Where("coin_id =?", chain.Coin3).And("amount > 0").Find(&userVirtualAmounts)
	if err != nil {
		return err, unassignedTotalCoinValue
	}

	//3、循环处理每个用户的数据(增加虚拟币，扣除静态贡献值)
	for _, item := range userVirtualAmounts {
		amount := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(item.Amount)) //用户贡献值余额
		getCoin := amount.Div(sumStaticValue).Mul(publishCoinValue)                        //得到的区块币
		needDestroyContribution := getCoin.Mul(platformGuidePriceForCoinValue)             //需销毁贡献值

		coinAmount, err := svc.GetUserCoinAmount(session, mid, chain.Coin2, item.Uid) //获取此时用户的"静态贡献值"
		if err != nil {
			return err, unassignedTotalCoinValue
		}
		coinAmountValue, _ := decimal.NewFromString(coinAmount)

		//3.1判断静态贡献值是否足够
		if needDestroyContribution.GreaterThan(coinAmountValue) {
			//TODO::公式【得到的区块币 - ((需销毁贡献值 - 用户贡献值余额) / 今日平台区块币指导价)】
			tempCoin := (needDestroyContribution.Sub(coinAmountValue)).Div(platformGuidePriceForCoinValue)
			getCoin = getCoin.Sub(tempCoin)
			unassignedTotalCoin.Add(getCoin)
			needDestroyContribution = coinAmountValue
		}

		//3.2给相应用户加上分配到的虚拟币
		err = DealUserCoin(session, md.DealUserCoinReq{
			Kind:         "add",
			Mid:          mid,
			Title:        md.DynamicAreaDistributionTitleForUserVirtualCoinFlow,
			TransferType: md.DynamicAreaDistributionTransferTypeForUserVirtualCoinFlow,
			OrdId:        "",
			CoinId:       chain.Coin1,
			Uid:          item.Uid,
			Amount:       zhios_order_relate_utils.StrToFloat64(getCoin.String()),
		})
		if err != nil {
			return err, unassignedTotalCoinValue
		}

		//3.3给相应用户扣除 "动态发放(销毁静态贡献值)"
		err = DealUserCoin(session, md.DealUserCoinReq{
			Kind:         "sub",
			Mid:          mid,
			Title:        md.DynamicIssueAndDestroyStaticContributionTitleForUserVirtualCoinFlow,
			TransferType: md.DynamicIssueAndDestroyStaticContributionTransferTypeForUserVirtualCoinFlow,
			OrdId:        "",
			CoinId:       chain.Coin2,
			Uid:          item.Uid,
			Amount:       zhios_order_relate_utils.StrToFloat64(needDestroyContribution.String()),
		})
		if err != nil {
			return err, unassignedTotalCoinValue
		}
	}

	//4、处理未分配完的区块币-动态区
	//unassignedTotalCoinValue = zhios_order_relate_utils.StrToFloat64(unassignedTotalCoin.String())
	if publishCoinValue.GreaterThan(unassignedTotalCoin) {
		//if unassignedTotalCoinValue > 0 {
		unassignedTotalCoinValue, _ = publishCoinValue.Sub(unassignedTotalCoin).Float64()
		err := DealDestroyCoin(session, int(enum.DynamicallyUnallocatedAndDestroy), unassignedTotalCoinValue, enum.DynamicallyUnallocatedAndDestroy.String(), &chain)
		if err != nil {
			return err, unassignedTotalCoinValue
		}
	}
	return nil, unassignedTotalCoinValue
}

/*
   统计分配区块币-运营中心
*/
func statisticsAndDistributeCoinForOperationCenter(session *xorm.Session, mid string, publishCoin float64, chain model.BlockStarChain) (err error, unassignedTotalCoinValue float64) {
	publishCoinValue := decimal.NewFromFloat(publishCoin) //运营中心-发行区块币数量
	var totalCoin = decimal.NewFromFloat(0)

	//1、查询出运营中心等级的所有用户
	var userList []*model.User
	totalUser, err := session.Table("user").Where("level =?", chain.OperationCenterForUserLevel).FindAndCount(&userList)
	if err != nil {
		return err, unassignedTotalCoinValue
	}
	if totalUser > 0 {
		singleValue := publishCoinValue.Div(decimal.NewFromInt(totalUser))
		//2、循环处理每个用户的数据(增加虚拟币)
		for _, item := range userList {
			totalCoin = totalCoin.Add(singleValue)
			err := DealUserCoin(session, md.DealUserCoinReq{
				Kind:         "add",
				Mid:          mid,
				Title:        md.OperationCenterTitleForUserVirtualCoinFlow,
				TransferType: md.OperationCenterTransferTypeForUserVirtualCoinFlow,
				OrdId:        "",
				CoinId:       chain.Coin1,
				Uid:          item.Uid,
				Amount:       zhios_order_relate_utils.StrToFloat64(singleValue.String()),
			})
			if err != nil {
				return err, unassignedTotalCoinValue
			}
		}
	}

	//3、处理未分配完的区块币-运营中心
	unassignedTotalCoinValue, _ = publishCoinValue.Sub(totalCoin).Float64()
	if unassignedTotalCoinValue > 0 {
		err := DealDestroyCoin(session, int(enum.OperationCenterUnallocatedAndDestroy), unassignedTotalCoinValue, enum.OperationCenterUnallocatedAndDestroy.String(), &chain)
		if err != nil {
			return err, unassignedTotalCoinValue
		}
	}
	return nil, unassignedTotalCoinValue
}

/*
   统计分配区块币-其他
*/
func statisticsAndDistributeCoinForOther(session *xorm.Session, mid string, publishCoin float64, chain model.BlockStarChain) (err error, unassignedTotalCoinValue float64) {
	publishCoinValue := decimal.NewFromFloat(publishCoin) //运营中心-发行区块币数量
	var totalCoin = decimal.NewFromFloat(0)

	var userLevel = make([]string, 0)
	err = json.Unmarshal([]byte(chain.OtherForUserLevel), &userLevel)
	if err != nil {
		return err, unassignedTotalCoinValue
	}
	if len(userLevel) > 0 {
		//1、查询出运营中心等级的所有用户
		var userList []*model.User
		totalUser, err := session.Table("user").In("level", userLevel).FindAndCount(&userList)
		if err != nil {
			return err, unassignedTotalCoinValue
		}
		if totalUser > 0 {
			singleValue := publishCoinValue.Div(decimal.NewFromInt(totalUser))
			//2、循环处理每个用户的数据(增加虚拟币)
			for _, item := range userList {
				totalCoin = totalCoin.Add(singleValue)
				err := DealUserCoin(session, md.DealUserCoinReq{
					Kind:         "add",
					Mid:          mid,
					Title:        md.OtherTitleForUserVirtualCoinFlow,
					TransferType: md.OtherTransferTypeForUserVirtualCoinFlow,
					OrdId:        "",
					CoinId:       chain.Coin1,
					Uid:          item.Uid,
					Amount:       zhios_order_relate_utils.StrToFloat64(singleValue.String()),
				})
				if err != nil {
					return err, unassignedTotalCoinValue
				}
			}
		}
	}

	//3、处理未分配完的区块币-其他
	unassignedTotalCoinValue, _ = publishCoinValue.Sub(totalCoin).Float64()
	if unassignedTotalCoinValue > 0 {
		err := DealDestroyCoin(session, int(enum.OtherUnallocatedAndDestroy), unassignedTotalCoinValue, enum.OtherUnallocatedAndDestroy.String(), &chain)
		if err != nil {
			return err, unassignedTotalCoinValue
		}
	}
	return nil, unassignedTotalCoinValue
}

//计算当前每日区块币应发行数量
func calcNowEverydayPublishCoin(session *xorm.Session, chain model.BlockStarChain) error {
	now := time.Now()
	startAt, err := time.ParseInLocation("2006-01-02", chain.StartAt, time.Local) //起始时间
	if err != nil {
		return err
	}
	diffDays := zhios_order_relate_utils.GetDiffDays(now, startAt) //相差天数
	remainder := diffDays % 30
	if remainder == 0 {
		everyThirtyDaysReduceRate := zhios_order_relate_utils.StrToFloat64(chain.EveryThirtyDaysReduceRate) / 100 //每三十日递减发行百分比
		nowEverydayPublishCoin := zhios_order_relate_utils.StrToFloat64(chain.NowEverydayPublishCoin)             //当前每日区块币发行数量
		chain.NowEverydayPublishCoin = zhios_order_relate_utils.Float64ToStrPrec10(nowEverydayPublishCoin * (1 - everyThirtyDaysReduceRate))
		updateAffected, err := db.BlockStarChainUpdate(session, chain.Id, &chain, "now_everyday_publish_coin")
		if err != nil {
			return err
		}
		if updateAffected == 0 {
			err = errors.New("更新 block_star_chain 的 now_everyday_publish_coin 记录失败")
			return err
		}
	}
	return nil
}

//处理虚拟币 - 发放
func dealIssueCoin(session *xorm.Session, kind int, amount float64, title string, chain *model.BlockStarChain) error {
	now := time.Now()
	var blockStarChainFlow model.BlockStarChainFlow
	blockStarChainFlow.CoinId = chain.Coin1
	blockStarChainFlow.Direction = md.FlowDirectionIncome
	blockStarChainFlow.Kind = kind
	blockStarChainFlow.Title = title
	blockStarChainFlow.CreateTime = now
	switch kind {
	case int(enum.SystemTimingIssue):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalNowCoin) + amount)
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalRemainderCoin) - amount)
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalPublishCoin) + amount)
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = chain.TotalDestroyCoin
		break
	}
	chain.TotalNowCoin = blockStarChainFlow.AfterTotalNowCoin
	chain.TotalRemainderCoin = blockStarChainFlow.AfterTotalRemainderCoin
	chain.TotalPublishCoin = blockStarChainFlow.AfterTotalPublishCoin
	chain.TotalDestroyCoin = blockStarChainFlow.AfterTotalDestroyCoin

	//更新 `block_star_chain` 表
	updateAffected, err := db.BlockStarChainUpdate(session, chain.Id, chain)
	if err != nil {
		return err
	}
	if updateAffected == 0 {
		err = errors.New("更新 block_star_chain 记录失败")
		return err
	}

	//插入 `block_star_chain_flow` 记录
	_, err = db.BlockStarChainFlowInsert(session, &blockStarChainFlow)
	if err != nil {
		return err
	}
	return nil
}

//DealDestroyCoin 处理虚拟币 - 销毁
func DealDestroyCoin(session *xorm.Session, kind int, amount float64, title string, chain *model.BlockStarChain) error {
	now := time.Now()
	var blockStarChainFlow model.BlockStarChainFlow
	blockStarChainFlow.CoinId = chain.Coin1
	blockStarChainFlow.Direction = md.FlowDirectionExpenditure
	blockStarChainFlow.Kind = kind
	blockStarChainFlow.Title = title
	blockStarChainFlow.CreateTime = now
	switch kind {
	case int(enum.SystemDestroy):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalRemainderCoin) - amount)
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalDestroyCoin) + amount)
		break
	case int(enum.StaticUnallocatedAndDestroy):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalNowCoin) - amount)
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalDestroyCoin) + amount)
		break
	case int(enum.DynamicallyUnallocatedAndDestroy):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalNowCoin) - amount)
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalDestroyCoin) + amount)
		break
	case int(enum.OperationCenterUnallocatedAndDestroy):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalNowCoin) - amount)
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalDestroyCoin) + amount)
		break
	case int(enum.OtherUnallocatedAndDestroy):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalNowCoin) - amount)
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalDestroyCoin) + amount)
		break
	case int(enum.RewardDestroy):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalNowCoin) - amount)
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalDestroyCoin) + amount)
		break
	case int(enum.RewardUnallocatedAndDestroy):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalNowCoin) - amount)
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalDestroyCoin) + amount)
		break
	case int(enum.GroupLotteryAndDestroy):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalNowCoin) - amount)
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalDestroyCoin) + amount)
		break
	case int(enum.RedEnvelopeTransferBalanceAndDestroy):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalNowCoin) - amount)
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalDestroyCoin) + amount)
		break
	case int(enum.TransferFeeAndDestroy):
		blockStarChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockStarChainFlow.AfterTotalNowCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalNowCoin) - amount)
		blockStarChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockStarChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockStarChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockStarChainFlow.AfterTotalDestroyCoin = zhios_order_relate_utils.Float64ToStrPrec10(zhios_order_relate_utils.StrToFloat64(chain.TotalDestroyCoin) + amount)
		break
	}

	chain.TotalNowCoin = blockStarChainFlow.AfterTotalNowCoin
	chain.TotalRemainderCoin = blockStarChainFlow.AfterTotalRemainderCoin
	chain.TotalPublishCoin = blockStarChainFlow.AfterTotalPublishCoin
	chain.TotalDestroyCoin = blockStarChainFlow.AfterTotalDestroyCoin
	//更新 `block_star_chain` 表
	updateAffected, err := db.BlockStarChainUpdate(session, chain.Id, chain)
	if err != nil {
		return err
	}
	if updateAffected == 0 {
		err = errors.New("更新 block_star_chain 记录失败")
		return err
	}

	//插入 `block_star_chain_flow` 记录
	_, err = db.BlockStarChainFlowInsert(session, &blockStarChainFlow)
	if err != nil {
		return err
	}
	return nil
}

// DealUserCoin 处理给用户虚拟币积分
func DealUserCoin(session *xorm.Session, req md.DealUserCoinReq) (err error) {
	//1、分布式锁阻拦
	requestIdPrefix := fmt.Sprintf(md.DealUserCoinRequestIdPrefix, req.Mid, req.CoinId, req.Uid)
	cb, err := svc.HandleDistributedLock(req.Mid, strconv.Itoa(req.Uid), requestIdPrefix)
	if err != nil {
		return err
	}
	if cb != nil {
		defer cb() // 释放锁
	}

	//2、计算&&组装数据
	now := time.Now()
	coinAmount, err := svc.GetUserCoinAmount(session, req.Mid, req.CoinId, req.Uid)
	if err != nil {
		return err
	}
	coinAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(coinAmount))
	amountValue := decimal.NewFromFloat(req.Amount)

	var userVirtualCoinFlow model.UserVirtualCoinFlow
	userVirtualCoinFlow.CoinId = req.CoinId
	userVirtualCoinFlow.Title = req.Title
	userVirtualCoinFlow.TransferType = req.TransferType
	userVirtualCoinFlow.Uid = req.Uid
	userVirtualCoinFlow.BeforeAmout = coinAmount
	userVirtualCoinFlow.Amout = amountValue.String()
	userVirtualCoinFlow.CreateTime = now

	if req.Kind == "add" {
		userVirtualCoinFlow.Direction = 1
		userVirtualCoinFlow.AfterAmout = coinAmountValue.Add(amountValue).String()
	} else if req.Kind == "sub" {
		userVirtualCoinFlow.Direction = 2
		userVirtualCoinFlow.AfterAmout = coinAmountValue.Sub(amountValue).String()
	} else {
		err = errors.New("错误的kind类型")
		return err
	}
	if zhios_order_relate_utils.StrToFloat64(userVirtualCoinFlow.AfterAmout) < 0 {
		return errors.New("用户虚拟币积分不足,无法扣除。")
	}
	//3、插入 `user_virtual_coin_flow` 记录
	_, err = db.UserVirtualCoinFlowInsert(session, &userVirtualCoinFlow)
	if err != nil {
		return err
	}

	//4、修改 `user_virtual_amount`的amount值 && 及缓存
	err = svc.SetCacheUserVirtualAmount(session, req.Mid, userVirtualCoinFlow.AfterAmout, req.CoinId, req.Uid, true)
	if err != nil {
		return err
	}

	return nil
}

// DealLotteryDraw 处理抽奖
func DealLotteryDraw(session *xorm.Session, req md.DealLotteryDrawReq) (err error) {
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()
	//1、查找 `block_star_chain` 基础设置
	blockStarChain, err := db.BlockStarChainGetOneByParams(session, map[string]interface{}{
		"key":   "is_use",
		"value": 1,
	})
	if err != nil {
		_ = session.Rollback()
		return err
	}
	rewardValue, _ := decimal.NewFromString(req.Amount)
	rewardValue = rewardValue.Mul(decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(blockStarChain.LotteryDrawDestroyCoinRate) / 100))
	platformGuidePriceForCoin, _ := decimal.NewFromString(blockStarChain.PlatformGuidePriceForCoin)
	needDestroyCoin := rewardValue.Div(platformGuidePriceForCoin) //须销毁的米豆
	destroyCoinValue, _ := needDestroyCoin.Float64()              //系统销毁 区块币值

	//2、用户扣除
	err = DealUserCoin(session, md.DealUserCoinReq{
		Kind:         "sub",
		Mid:          req.Mid,
		Title:        md.GroupLotteryAndDestroyTitleForUserVirtualCoinFlow,
		TransferType: md.GroupLotteryAndDestroyTransferTypeForUserVirtualCoinFlow,
		OrdId:        "",
		CoinId:       blockStarChain.Coin1,
		Uid:          req.Uid,
		Amount:       destroyCoinValue,
	})
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//3、系统销毁
	err = DealDestroyCoin(session, int(enum.GroupLotteryAndDestroy), destroyCoinValue, enum.GroupLotteryAndDestroy.String(), blockStarChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败")
	}
	return nil
}
