package svc

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"encoding/json"
	"time"
	"xorm.io/xorm"
)

//加入区域代理直推上级分佣 抽成部分
/***
commission 总佣金
bili	抽成 除以100之后的
uid	 用户id
oid 订单号
pvd 渠道
*/
func AddExtendAgentCommission(eg *xorm.Engine, args map[string]string) {
	user, err := db.UserProfileFindByID(eg, args["uid"])
	if err != nil || user == nil {
		return
	}
	if user.ParentUid == 0 {
		return
	}
	//判断是不是代理
	//查出用户是否区域代理
	agentUser, err := db.GetCountByRegionalAgentUserInfoByParams(eg, map[string]interface{}{
		"key":   "uid",
		"value": user.Uid,
	})
	if agentUser == nil || err != nil {
		return
	}
	// 有效期计算
	if agentUser.IsFailure == 1 {
		return
	}
	now := time.Now().Unix()
	if agentUser.FailureTime > 0 && agentUser.FailureTime < int(now) {
		return
	}

	commissionOpt, err := db.DbsPlanCommissionById(eg, zhios_order_relate_utils.StrToInt(args["plan_commission_id"]))
	if err != nil || commissionOpt == nil || commissionOpt.Id == 0 {
		return
	}
	var subsidyTmp = make(map[int]*md.ExtendRewardLvGrade)
	var tmp map[int]*md.ExtendRewardLvGrade
	if err := json.Unmarshal([]byte(commissionOpt.Data), &tmp); err != nil {
		return
	}
	for _, v := range tmp {
		subsidyTmp[v.Lv] = v
	}
	userProfile, err := db.UserProfileFindByID(eg, args["uid"])
	if err != nil || userProfile == nil {
		return
	}
	if userProfile.ParentUid == 0 {
		return
	}
	//查询用户等级
	parentUser, err := db.UserFindByID(eg, userProfile.ParentUid)
	if err != nil || parentUser == nil {
		return
	}
	lvGrade := subsidyTmp[parentUser.Level]
	var rewardList = make(map[string]md.ExtendPriceRewardListDetail)
	if agentUser.Level == 1 {
		rewardList = lvGrade.ExtendPriceRewardBuyGoodsProvince
	}
	if agentUser.Level == 2 {
		rewardList = lvGrade.ExtendPriceRewardBuyGoodsCity
	}
	if agentUser.Level == 3 {
		rewardList = lvGrade.ExtendPriceRewardBuyGoodsArea
	}
	//判断有没有存过
	relate, _ := db.GetExtendAgentCommissionRelate(eg, args["oid"], args["pvd"])
	if relate != nil {
		return
	}

	for k, v := range rewardList {
		var amount float64 = 0
		amount = zhios_order_relate_utils.StrToFloat64(args["commission"]) * zhios_order_relate_utils.StrToFloat64(v.Bili) / 100
		if amount == 0 {
			continue
		}
		var ord = model.ExtendAgentCommissionRelate{
			Uid:        user.ParentUid,
			FormUid:    user.Uid,
			Pvd:        args["pvd"],
			Oid:        args["oid"],
			CreateTime: time.Now(),
			Amount:     zhios_order_relate_utils.Float64ToStrByPrec(amount, 4),
			Rate:       args["bili"],
			CoinId:     zhios_order_relate_utils.StrToInt(k),
		}
		eg.InsertOne(&ord)
	}

	return
}

//结算分佣抽成部分
/***
oid 订单号
pvd 渠道
title 描述
*/
func SettleExtendUidCommission(session *xorm.Session, args map[string]string) bool {
	relate, err := db.GetExtendAgentCommissionRelateWithSession(session, args["oid"], args["pvd"], "0")
	if err != nil {
		return false
	}
	if relate == nil {
		return true
	}
	if relate.SettleTime.IsZero() == false {
		return true
	}
	user, _ := db.UserProfileFindByIDWithSession(session, relate.Uid)
	amount := zhios_order_relate_utils.StrToFloat64(relate.Amount)
	mBeforeAmount := zhios_order_relate_utils.StrToFloat64(user.FinValid)
	mAfterAmount := mBeforeAmount + amount
	//
	now := time.Now()
	var finUserFlow = model.FinUserFlow{
		Type:         0,
		Uid:          relate.Uid,
		Amount:       zhios_order_relate_utils.AnyToString(amount),
		BeforeAmount: zhios_order_relate_utils.AnyToString(mBeforeAmount),
		AfterAmount:  zhios_order_relate_utils.AnyToString(mAfterAmount),
		OrdType:      "own",
		OrdId:        zhios_order_relate_utils.AnyToString(relate.Oid),
		OrdAction:    14,
		OrdDetail:    zhios_order_relate_utils.AnyToString(relate.Oid),
		State:        2,
		OrdTitle:     "推荐代理购买商品",
		OrdTime:      int(now.Unix()),
		CreateAt:     now,
		UpdateAt:     now,
	}
	user.FinValid = zhios_order_relate_utils.Float64ToStrByPrec(mAfterAmount, 4)
	affected, err := session.Where("uid=?", user.Uid).Cols("fin_valid").Update(user)
	has, errs := session.Insert(&finUserFlow)
	if affected == 0 || err != nil || errs != nil || has == 0 {
		_ = zhios_order_relate_logx.Warn(err)
		return false
	}
	relate.SettleTime = time.Now()
	update, err := session.Where("id=?", relate.Id).Cols("settle_time").Update(relate)
	if update == 0 || err != nil {
		_ = zhios_order_relate_logx.Warn(err)
		return false
	}
	t := time.Now()
	//
	_, err = session.Insert(&model.MarkSettleRecord{
		Uid:        relate.Uid,
		Amount:     zhios_order_relate_utils.Float64ToStrByPrec(amount, 4),
		Time:       t,
		Type:       "extend_agent_commission",
		CoinId:     0,
		BelongTime: t.Format("2006-01"),
		Pvd:        args["pvd"],
		Oid:        args["oid"],
	})
	if err != nil {
		_ = zhios_order_relate_logx.Warn(err)
		return false
	}
	return true
}
